@@ -316,7 +316,7 @@ sub sendmail {
         From    => $self->mailfrom || 'dimka@uvw.ru',
         To      => $self->mailto || 'dimka@uvw.ru',
         Subject => $subject,
-        type    => 'multipart/fixed',
+        Type    => 'multipart/fixed',
     );
 
     local $Data::Dumper::Indent = 1;
@@ -327,8 +327,15 @@ sub sendmail {
 
 
     $mail->attach(
-        Type    => 'text/plain; charset=utf-8',
-        Data    => Dumper($task->data),
+        Type        => 'text/plain; charset=utf-8',
+        Disposition => 'inline',
+        Data        => encode_utf8($error),
+    );
+    $mail->attach(
+        Type        => 'text/plain; charset=utf-8',
+        Filename    => 'task.dump.txt',
+        Disposition => 'inline',
+        Data        => Dumper($task),
     );
 
     $mail->add($_ => $self->mailheaders->{$_}) for keys %{ $self->mailheaders };
